package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.vo.TreeSelect;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import java.util.ArrayList;
import com.ruoyi.system.domain.vo.MetaVo;
import java.util.LinkedList;
import com.ruoyi.system.domain.vo.RouterVo;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.common.security.utils.SecurityUtils;
import java.util.Iterator;
import java.util.Collection;
import java.util.Arrays;
import com.ruoyi.common.core.utils.StringUtils;
import java.util.HashSet;
import java.util.Set;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.domain.SysMenu;
import java.util.List;
import com.ruoyi.system.mapper.SysRoleMenuMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SysMenuMapper;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISysMenuService;

@Service
public class SysMenuServiceImpl implements ISysMenuService
{
    public static final String PREMISSION_STRING = "perms[\"{0}\"]";
    @Autowired
    private SysMenuMapper menuMapper;
    @Autowired
    private SysRoleMapper roleMapper;
    @Autowired
    private SysRoleMenuMapper roleMenuMapper;
    
    public List<SysMenu> selectMenuList(final Long userId) {
        return this.selectMenuList(new SysMenu(), userId);
    }
    
    public List<SysMenu> selectMenuList(final SysMenu menu, final Long userId) {
        List<SysMenu> menuList = null;
        if (SysUser.isAdmin(userId)) {
            menuList = this.menuMapper.selectMenuList(menu);
        }
        else {
            menu.getParams().put("userId", userId);
            menuList = this.menuMapper.selectMenuListByUserId(menu);
        }
        return menuList;
    }
    
    public Set<String> selectMenuPermsByUserId(final Long userId) {
        final List<String> perms = this.menuMapper.selectMenuPermsByUserId(userId);
        final Set<String> permsSet = new HashSet<String>();
        for (final String perm : perms) {
            if (StringUtils.isNotEmpty(perm)) {
                permsSet.addAll(Arrays.asList(perm.trim().split(",")));
            }
        }
        return permsSet;
    }
    
    public Set<String> selectMenuPermsByRoleId(final Long roleId) {
        final List<String> perms = this.menuMapper.selectMenuPermsByRoleId(roleId);
        final Set<String> permsSet = new HashSet<String>();
        for (final String perm : perms) {
            if (StringUtils.isNotEmpty(perm)) {
                permsSet.addAll(Arrays.asList(perm.trim().split(",")));
            }
        }
        return permsSet;
    }
    
    public List<SysMenu> selectMenuTreeByUserId(final Long userId) {
        List<SysMenu> menus = null;
        if (SecurityUtils.isAdmin(userId)) {
            menus = this.menuMapper.selectMenuTreeAll();
        }
        else {
            menus = this.menuMapper.selectMenuTreeByUserId(userId);
        }
        return this.getChildPerms(menus, 0);
    }
    
    public List<Long> selectMenuListByRoleId(final Long roleId) {
        final SysRole role = this.roleMapper.selectRoleById(roleId);
        return this.menuMapper.selectMenuListByRoleId(roleId, role.isMenuCheckStrictly());
    }
    
    public List<RouterVo> buildMenus(final List<SysMenu> menus) {
        final List<RouterVo> routers = new LinkedList<RouterVo>();
        for (final SysMenu menu : menus) {
            final RouterVo router = new RouterVo();
            router.setHidden("1".equals(menu.getVisible()));
            router.setName(this.getRouteName(menu));
            router.setPath(this.getRouterPath(menu));
            router.setComponent(this.getComponent(menu));
            router.setQuery(menu.getQuery());
            router.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StringUtils.equals((CharSequence)"1", (CharSequence)menu.getIsCache()), menu.getPath()));
            final List<SysMenu> cMenus = menu.getChildren();
            if (StringUtils.isNotEmpty((Collection)cMenus) && "M".equals(menu.getMenuType())) {
                router.setAlwaysShow(Boolean.valueOf(true));
                router.setRedirect("noRedirect");
                router.setChildren((List)this.buildMenus(cMenus));
            }
            else if (this.isMenuFrame(menu)) {
                router.setMeta((MetaVo)null);
                final List<RouterVo> childrenList = new ArrayList<RouterVo>();
                final RouterVo children = new RouterVo();
                children.setPath(menu.getPath());
                children.setComponent(menu.getComponent());
                children.setName(StringUtils.capitalize(menu.getPath()));
                children.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StringUtils.equals((CharSequence)"1", (CharSequence)menu.getIsCache()), menu.getPath()));
                children.setQuery(menu.getQuery());
                childrenList.add(children);
                router.setChildren((List)childrenList);
            }
            else if (menu.getParentId().intValue() == 0 && this.isInnerLink(menu)) {
                router.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon()));
                router.setPath("/");
                final List<RouterVo> childrenList = new ArrayList<RouterVo>();
                final RouterVo children = new RouterVo();
                final String routerPath = this.innerLinkReplaceEach(menu.getPath());
                children.setPath(routerPath);
                children.setComponent("InnerLink");
                children.setName(StringUtils.capitalize(routerPath));
                children.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), menu.getPath()));
                childrenList.add(children);
                router.setChildren((List)childrenList);
            }
            routers.add(router);
        }
        return routers;
    }
    
    public List<SysMenu> buildMenuTree(final List<SysMenu> menus) {
        List<SysMenu> returnList = new ArrayList<SysMenu>();
        final List<Long> tempList = menus.stream().map((Function<? super Object, ?>)SysMenu::getMenuId).collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
        for (final SysMenu menu : menus) {
            if (!tempList.contains(menu.getParentId())) {
                this.recursionFn(menus, menu);
                returnList.add(menu);
            }
        }
        if (returnList.isEmpty()) {
            returnList = menus;
        }
        return returnList;
    }
    
    public List<TreeSelect> buildMenuTreeSelect(final List<SysMenu> menus) {
        final List<SysMenu> menuTrees = this.buildMenuTree(menus);
        return menuTrees.stream().map((Function<? super Object, ?>)TreeSelect::new).collect((Collector<? super Object, ?, List<TreeSelect>>)Collectors.toList());
    }
    
    public SysMenu selectMenuById(final Long menuId) {
        return this.menuMapper.selectMenuById(menuId);
    }
    
    public boolean hasChildByMenuId(final Long menuId) {
        final int result = this.menuMapper.hasChildByMenuId(menuId);
        return result > 0;
    }
    
    public boolean checkMenuExistRole(final Long menuId) {
        final int result = this.roleMenuMapper.checkMenuExistRole(menuId);
        return result > 0;
    }
    
    public int insertMenu(final SysMenu menu) {
        return this.menuMapper.insertMenu(menu);
    }
    
    public int updateMenu(final SysMenu menu) {
        return this.menuMapper.updateMenu(menu);
    }
    
    public int deleteMenuById(final Long menuId) {
        return this.menuMapper.deleteMenuById(menuId);
    }
    
    public boolean checkMenuNameUnique(final SysMenu menu) {
        final Long menuId = StringUtils.isNull((Object)menu.getMenuId()) ? -1L : menu.getMenuId();
        final SysMenu info = this.menuMapper.checkMenuNameUnique(menu.getMenuName(), menu.getParentId());
        return !StringUtils.isNotNull((Object)info) || info.getMenuId() == (long)menuId;
    }
    
    public String getRouteName(final SysMenu menu) {
        String routerName = StringUtils.capitalize(menu.getPath());
        if (this.isMenuFrame(menu)) {
            routerName = "";
        }
        return routerName;
    }
    
    public String getRouterPath(final SysMenu menu) {
        String routerPath = menu.getPath();
        if (menu.getParentId().intValue() != 0 && this.isInnerLink(menu)) {
            routerPath = this.innerLinkReplaceEach(routerPath);
        }
        if (0 == menu.getParentId().intValue() && "M".equals(menu.getMenuType()) && "1".equals(menu.getIsFrame())) {
            routerPath = "/" + menu.getPath();
        }
        else if (this.isMenuFrame(menu)) {
            routerPath = "/";
        }
        return routerPath;
    }
    
    public String getComponent(final SysMenu menu) {
        String component = "Layout";
        if (StringUtils.isNotEmpty(menu.getComponent()) && !this.isMenuFrame(menu)) {
            component = menu.getComponent();
        }
        else if (StringUtils.isEmpty(menu.getComponent()) && menu.getParentId().intValue() != 0 && this.isInnerLink(menu)) {
            component = "InnerLink";
        }
        else if (StringUtils.isEmpty(menu.getComponent()) && this.isParentView(menu)) {
            component = "ParentView";
        }
        return component;
    }
    
    public boolean isMenuFrame(final SysMenu menu) {
        return menu.getParentId().intValue() == 0 && "C".equals(menu.getMenuType()) && menu.getIsFrame().equals("1");
    }
    
    public boolean isInnerLink(final SysMenu menu) {
        return menu.getIsFrame().equals("1") && StringUtils.ishttp(menu.getPath());
    }
    
    public boolean isParentView(final SysMenu menu) {
        return menu.getParentId().intValue() != 0 && "M".equals(menu.getMenuType());
    }
    
    public List<SysMenu> getChildPerms(final List<SysMenu> list, final int parentId) {
        final List<SysMenu> returnList = new ArrayList<SysMenu>();
        for (final SysMenu t : list) {
            if (t.getParentId() == parentId) {
                this.recursionFn(list, t);
                returnList.add(t);
            }
        }
        return returnList;
    }
    
    private void recursionFn(final List<SysMenu> list, final SysMenu t) {
        final List<SysMenu> childList = this.getChildList(list, t);
        t.setChildren((List)childList);
        for (final SysMenu tChild : childList) {
            if (this.hasChild(list, tChild)) {
                this.recursionFn(list, tChild);
            }
        }
    }
    
    private List<SysMenu> getChildList(final List<SysMenu> list, final SysMenu t) {
        final List<SysMenu> tlist = new ArrayList<SysMenu>();
        for (final SysMenu n : list) {
            if (n.getParentId() == (long)t.getMenuId()) {
                tlist.add(n);
            }
        }
        return tlist;
    }
    
    private boolean hasChild(final List<SysMenu> list, final SysMenu t) {
        return this.getChildList(list, t).size() > 0;
    }
    
    public String innerLinkReplaceEach(final String path) {
        return StringUtils.replaceEach(path, new String[] { "http://", "https://", "www.", ".", ":" }, new String[] { "", "", "", "/", "/" });
    }
}
